/*****************************************************************************
# Copyright (C) 1994 X Consortium
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
# AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNEC-
# TION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# Except as contained in this notice, the name of the X Consortium shall not
# be used in advertising or otherwise to promote the sale, use or other deal-
# ings in this Software without prior written authorization from the X Consor-
# tium.
#
#
# FSF changes to this file are in the public domain.
***************************************************************************/


#include <iostream>
using namespace std;
#include <sys/resource.h>
#include "eo"
#include <unistd.h>
#include "utils/eoDistance.h"
#include "utils/eoRealVectorBounds.h"

#include "Individu/individu.h"
#include "Initialisation/aveugle.h"
#include "Croisement/croise.h"
#include "Mutations/ajoutDeStationSansInit.h"
#include "Mutations/V1.h"
#include "Mutations/delStation.h"
#include "Mutations/delFluent.h"
#include "Stat/stat.h"
#include "Fitness/progressiveEvaluation.h"



typedef eoMinimizingFitness MyFitT ;	
typedef eoCPTEO_V1<MyFitT> Indi;      


#include <make_pop.h>
eoPop<Indi >&  make_pop(eoParser& _parser, eoState& _state, eoInit<Indi> & _init)
{
  return do_make_pop(_parser, _state, _init);
}


#include <make_continue.h>
eoContinue<Indi>& make_continue(eoParser& _parser, eoState& _state, eoEvalFuncCounter<Indi> & _eval)
{
  return do_make_continue(_parser, _state, _eval);
}


#include <make_checkpoint.h>
eoCheckPoint<Indi>& make_checkpoint(eoParser& _parser, eoState& _state, eoEvalFuncCounter<Indi>& _eval, eoContinue<Indi>& _continue)
{
  return do_make_checkpoint(_parser, _state, _eval, _continue);
}

#include <make_algo_scalar.h>
eoAlgo<Indi>&  make_algo_scalar(eoParser& _parser, eoState& _state, eoEvalFunc<Indi>& _eval, eoContinue<Indi>& _continue, eoGenOp<Indi>& _op, eoDistance<Indi> *_dist = NULL)
{
  return do_make_algo_scalar(_parser, _state, _eval, _continue, _op, _dist);
}

#include <make_run.h>

#include <eoScalarFitness.h>
void run_ea(eoAlgo<Indi>& _ga, eoPop<Indi>& _pop)
{
  do_run(_ga, _pop);
}


void make_help(eoParser & _parser);

int mainVirtual(int argc, char* argv[]){
	//VariablesGlobales var;
	
	
	
	time_t start,end;
  	double duree;
  	time(&start);
try
  {
    eoParser parser(argc, argv);  
	global.initialisation(parser);
	eoState state;    
   
    //eoCPTEO_V1EvalFunc<Indi> plainEval;
    
   	progressiveEvaluation<Indi> plainEval;
    eoEvalFuncCounter<Indi> eval(plainEval);

    
    eoCPTEO_V1Init<Indi> init;
   
    
    eoCPTEO_V1QuadCrossover<Indi> cross ;

    
  	eoCPTEO_V1Mutation<Indi> mut1 ;
	  double mut1Rate;   
	 
	 if(global.getPredicateSize()==global.problemGoal.size()) mut1Rate = parser.createParam(0, "mut1Rate", "Relative rate for atoms modification mutation 1", '1', "Variation Operators").value();
	 else mut1Rate = parser.createParam(35.0, "mut1Rate", "Relative rate for atoms modification mutation 1", '1', "Variation Operators").value();

    // create the combined operator with the first one (rename it cross1 !!!)
	eoPropCombinedMonOp<Indi> mut(mut1, mut1Rate);

    // and as many as you want the following way:
    // 1- write the new class by mimicking eoCPTEO_V1Mutation.h
    // 2- include that file here together with eoCPTEO_V1Mutation above
    // 3- uncomment and duplicate the following lines:
    //
	// eoCPTEO_V1AddMutation<Indi> mut2;
	 ajoutDeStation<Indi> mut2;
	double mut2Rate = parser.createParam(35.0, "mut2Rate", "Relative rate for add station mutation", '2', "Variation Operators").value();
	mut.add(mut2, mut2Rate);
	
	eoCPTEO_V1DelMutation<Indi> mut3;
	double mut3Rate = parser.createParam(3.0, "mut3Rate", "Relative rate for del station mutation", '2', "Variation Operators").value();
	mut.add(mut3, mut3Rate);
	
	eoCPTEO_V1DelFluentMutation<Indi> mut4;
	double mut4Rate = parser.createParam(7.0, "mut4Rate", "Relative rate for del atoms mutation", '2', "Variation Operators").value();

  // NOTE: if you want some gentle output, the last one shoudl be like
    mut.add(mut4, mut4Rate, true);
	
    
    double pCross = parser.createParam(0.3, "pCross", "Probability of Crossover", 'C', "Variation Operators" ).value();
    
    if ( (pCross < 0) || (pCross > 1) )
      throw runtime_error("Invalid pCross");

    double pMut = parser.createParam(0.8, "pMut", "Probability of Mutation", 'M', "Variation Operators" ).value();
    
    if ( (pMut < 0) || (pMut > 1) )
      throw runtime_error("Invalid pMut");

    
    eoSGAGenOp<Indi> op(cross, pCross, mut, pMut);

   eoPop<Indi>& pop   = make_pop(parser, state, init);

   eoContinue<Indi> & term = make_continue(parser, state, eval);

  eoCheckPoint<Indi> & checkpoint = make_checkpoint(parser, state, eval, term);

  eoCPTEO_V1Stat<Indi>   myStat;       
  checkpoint.add(myStat);
 
  eoIncrementorParam<unsigned> generationCounter("Gen.");
  checkpoint.add(generationCounter);
  
  std::string dirName =  parser.getORcreateParam(std::string("Res"), "resDir", "Directory to store DISK outputs", '\0', "Output - Disk").value() + "/";


  
  eoStdoutMonitor *myStdOutMonitor;
  eoFileMonitor   *myFileMonitor;
#ifdef HAVE_GNUPLOT
  eoGnuplot1DMonitor *myGnuMonitor;
#endif

  
  bool printCPTEO_V1Stat = parser.createParam(false, "coutCPTEO_V1Stat", "Prints my stat to screen, one line per generation", '\0', "My application").value();
  bool fileCPTEO_V1Stat = parser.createParam(false, "fileCPTEO_V1Stat", "Saves my stat to file (in resDir", '\0', "My application").value();
  bool plotCPTEO_V1Stat = parser.createParam(false, "plotCPTEO_V1Stat", "On-line plots my stat using gnuplot", '\0', "My application").value();

  
  if (printCPTEO_V1Stat)
    {
      myStdOutMonitor = new eoStdoutMonitor(false);
     
      state.storeFunctor(myStdOutMonitor);
      
      checkpoint.add(*myStdOutMonitor);
      
      myStdOutMonitor->add(generationCounter);
      myStdOutMonitor->add(eval);
      myStdOutMonitor->add(myStat);
    }

  
  if (fileCPTEO_V1Stat || plotCPTEO_V1Stat)
      if (! testDirRes(dirName, true) )
	throw runtime_error("Problem with resDir");

 
  if (fileCPTEO_V1Stat)
    {
      
      myFileMonitor = new eoFileMonitor(dirName + "myStat.xg");
      // don't forget to store the memory in the state
      state.storeFunctor(myFileMonitor);
      // and of course to add the monitor to the checkpoint
      checkpoint.add(*myFileMonitor);
      // and the different fields to the monitor
      myFileMonitor->add(generationCounter);
      myFileMonitor->add(eval);
      myFileMonitor->add(myStat);
    }


#ifdef HAVE_GNUPLOT
  // should we PLOT it on StdOut ? (one dot per generation, incremental plot)
  if (plotCPTEO_V1Stat)
    {
      myGnuMonitor = new eoGnuplot1DMonitor(dirName+"plot_myStat.xg",minimizing_fitness<Indi>());
      // NOTE: you cand send commands to gnuplot at any time with the method
      // myGnuMonitor->gnuplotCommand(string)
      // par exemple, gnuplotCommand("set logscale y")

      // don't forget to store the memory in the state
      state.storeFunctor(myGnuMonitor);
      // and of course to add the monitor to the checkpoint
      checkpoint.add(*myGnuMonitor);
      // and the different fields to the monitor (X = eval, Y = myStat)
      myGnuMonitor->add(eval);
      myGnuMonitor->add(myStat);
    }
#endif

  // algorithm (need the operator!)
  eoAlgo<Indi>& ga = make_algo_scalar(parser, state, eval, checkpoint, op);
 // distanceFit<Indi> dist;
  //eoFitnessDistance<Indi> dist;
  // and the distance if you want to do sharing
  // eoAlgo<Indi>& ga = make_algo_scalar(parser, state, eval, checkpoint, op, &dist);

  ///// End of construction of the algorithm

    make_help(parser);
     eoParam* param=parser.getParamWithLongName("seed");
    __seed=param->getValue();

  //// GO
  ///////
  // evaluate intial population AFTER help and status in case it takes time
  apply<Indi>(eval, pop);
  	double fitnessTest=1e60;
  	for(int k=0;k<11;k++){ 
		
 	 	run_ea(ga, pop); // run the ga
    	
    	if(minFitness!=0 && fitnessTest>minFitness){
			
			time(&end);
  			duree=difftime(end,start);
  			result=fopen(resultFile.c_str(),"wb");
  			fprintf(result,";Total run Time : %.2f\n", duree);
			pop.best_element().printPlanVal(result);
  			fclose(result);
  			fitnessTest=minFitness;
  			cout << "The best is  " << pop.best_element() << "\n\n";
		}
  		
	}

  }
  catch(exception& e)
  {
    cout << e.what() << endl;
  }
  	time(&end);
	duree=difftime(end,start);
	cout << "duree de la campagne de test "<< duree << "\n";
  return 0;
}



int main(int argc, char* argv[])
{
	
	
	std::string sav1,sav2,sav3; 
	if(argc>1){
		for(int i=0;i<argc;i++){
			if(strcmp(argv[i],"-f")==0 && (i+1)<argc){
				sav1 ="--f=";
				sav1 += argv[i+1];
				argv[i]=(char *)sav1.c_str();
				for(int j=i+1;j<argc-1;j++){
					argv[j]=argv[j+1];	
				}
			}
			if(strcmp(argv[i],"-o")==0 && (i+1)<argc){
				sav2 ="--o=";
				sav2 += argv[i+1];
				argv[i]=(char *)sav2.c_str();
				for(int j=i+1;j<argc-1;j++){
					argv[j]=argv[j+1];	
				}
					
			}
			
		}
		if(argc > 4){
				sav3="@parametre";
				 argv[argc-1]=(char *)sav3.c_str();
			}		
	}else {
                sav1=+argv[0];
		cout << "Usage:   "+sav1 +" -o <domain file> -f <problem file> [Options] [@<parameters file>] \n";
		cout << "     or  "+sav1 +" @<parameters file> \n";
		cout <<"Input: \n" <<"   -o  <file>\n"<<"       Domain file (typed/untyped PDDL)\n"<<"   -f <file>\n"<<"       Problem file (typed/untyped PDDL)\n";
		cout <<"   @<parameters file>\n"<<"       List of all parameters (please read the generated file <bin>.status)\n";
		eoParser parser(argc, argv); 
		eoParser parser1(argc, argv);
		global.help(parser);
		 eoState state;    
      	progressiveEvaluation<Indi> plainEval;
   		eoEvalFuncCounter<Indi> eval(plainEval);
	    eoCPTEO_V1Init<Indi> init;
        eoCPTEO_V1QuadCrossover<Indi> cross ;
		eoContinue<Indi> & term = make_continue(parser, state, eval);
		eoCheckPoint<Indi> & checkpoint = make_checkpoint(parser, state, eval, term);
		eoCPTEO_V1Mutation<Indi> mut1 ;
	    double mut1Rate = parser1.createParam(35.0, "mut1Rate", "Relative rate for mutation 1", '1', "Variation Operators").value();
		eoPropCombinedMonOp<Indi> mut(mut1, mut1Rate);
		eoSGAGenOp<Indi> op(cross, 0.4, mut, 0.9);
		 make_algo_scalar(parser, state, eval, checkpoint, op);
		make_help(parser);
		return 0;
	}

try
  {
	mainVirtual(argc,argv);
  } 
  catch(exception& e)
  {
    cout << e.what() << endl;
    return 0;
  }
	//return 0;
		
}
