#ifndef AJOUTDESTATIONSANSINIT_H_
#define AJOUTDESTATIONSANSINIT_H_

/**
 * @class eoCPTEO_V1AddMutation
 * @author Jacques Bibai
 * @brief definition d'un operateur de mutation : mutation permettant l'ajout d'une
 * gare.  
 */
template<class GenotypeT> 
class ajoutDeStation: public eoMonOp<GenotypeT>{
public:
	 /**
		 * Constructeur vide
		 */
  ajoutDeStation()
  {
    
  }
  
 
  
	string className() const { return "ajoutDeStation"; }

  /**
   * @fn bool operator()(GenotypeT &)
   * @brief Operateur decrivant la maniere d'effectuer une mutation par ajout de gare. 
   * Ici on tire aleatoirement une gare. Ensuite on insere cette une gare construite a cette place.  
   * @param individu instance de la classe GenotypeT
   * @return bool : vrai si l'operation a reussi et faux sinon
   */ 
  bool operator()(GenotypeT & _genotype)
  {

     bool isModified=false;
     unsigned which,pos=0;
	 which =rng.random( _genotype.getBreakStation());
	 Gene station;
	   
    if(which>0 && which < _genotype.getGenome().size() )
    { 
    	//sauvegarde et initialisation du genotype
    	// definition des ensembles mutex, intersection et difference
    	std::vector<int> mutexF;
    	std::vector<int> mutexS;
    	std::vector<int> intersection;
    	std::vector<int> minus;
    	mutexF.resize(_genotype.get(which-1).size()>_genotype.get(which).size()?_genotype.get(which).size():_genotype.get(which-1).size());
		mutexS.resize(_genotype.get(which-1).size()>_genotype.get(which).size()?_genotype.get(which).size():_genotype.get(which-1).size());
		intersection.resize(_genotype.get(which-1).size()>_genotype.get(which).size()?_genotype.get(which).size():_genotype.get(which-1).size());
		minus.resize(_genotype.get(which).size());
		//definition et initialisation des variables contenant les tailles reelles des differents ensembles
		int fMutex=0,inter=0,moins=0;
		//initialisation des differents ensembles
		for(size_t i=0;i<_genotype.get(which).size();i++)
		{
			if(_genotype.get(which-1).mutex(_genotype.get(which).get(i),_genotype.get(which-1).size()))
			{
				mutexF[fMutex]=_genotype.get(which-1).mutex(_genotype.get(which).get(i),_genotype.get(which-1).size());
				mutexS[fMutex]=_genotype.get(which).mutex(mutexF[fMutex],_genotype.get(which).size());
				fMutex++;
			}
			else if(_genotype.get(which-1).exist(_genotype.get(which).get(i))>0)
				{
					intersection[inter]=_genotype.get(which).get(i);
					inter++;
				}
				else
				{
					minus[moins]=_genotype.get(which).get(i);
					moins++;
				}
		}
		//verification de l'egalite entre les deux stations
		if(fMutex==0 && moins==0) return false;
		//sauvegarde des gares 
    	std::vector<Gene> tmp;
	    tmp.resize(_genotype.getGenome().size());
	    for(size_t i=0;i<tmp.size();i++){
		   		tmp[i].getGene().resize(_genotype.get(i).size());
		   		 for(size_t j=0;j<tmp[i].size();j++){
		   		 	tmp[i].set(j,_genotype.get(i).get(j));
		   		 }
		   		 _genotype.get(i).getGene().clear();
		   	}
	     _genotype.getGenome().clear();
		 _genotype.getGenome().resize(tmp.size()+1);
		 // fin de la reinitialisation et sauvegarde du genotype  
	     /*******************************************************
	      *														* 
	      * debut de construction de la station intermediare	*
	      * 													*
	      *******************************************************/
		   // choix de la taille 
			//station.getGene().resize(inter+fMutex+rng.random(moins));
			// fin du choix de la taille
			//construction de la station
			//fluents de l'intersection 
			for(size_t i=0;i<(size_t)inter;i++){
			   	   		station.getGene().push_back(intersection[i]);
			   }
			   
			 for(size_t i=0;i<(size_t)fMutex;i++)
			   {
			   		if(rng.flip(0.5) && station.mutex2(mutexF[i],station.size())==0
					 && station.notExist(mutexF[i]))station.getGene().push_back(mutexF[i]);
			   		else if(station.mutex2(mutexS[i],station.size())==0
			   		&& station.notExist(mutexS[i])) station.getGene().push_back(mutexS[i]);
			   }  
			     
			 for(size_t i=0;(int)i<moins;i++)
			   {
			   		if(rng.flip(0.5)&& station.mutex2(minus[i],station.size())==0
					 && station.notExist(minus[i]))station.getGene().push_back(minus[i]);
			   		
			   		
			   }    
			 int ho;
	 	if(rng.flip(global.getpMeanAveragePerMutation()/tmp.size())){
	 		CPTPredicateArray preds = cptGetPredicates();
  			std::vector<unsigned>jeu;
  			jeu.resize(global.getPredicateSize());
  			//initialisation avec la valeur exacte de la table des predicats
  			for(size_t i=0;i<jeu.size();i++) 
  			{	size_t j;
  				for(j=0;j<(unsigned)preds.nb;j++)
  					if(strcmp(cptGetPredicateName(preds.elems[j]),global.getPredicate(i).c_str())==0)break;
  				jeu[i]=j;
  			}
  			unsigned posPredicate, plus=eo::rng.random(global.getDistance());
  			bool test;
  			for(size_t j=0;j<plus;j++)
			{
				ho=0;
				do
				{
					test=false;
					//choix du predicat 
					posPredicate=rng.random(jeu.size());
					//debut tirage aleatoire de termes 
					//	cout << posPredicate << "***************\n";				
					std::vector<CPTTermArray> terms;
			    	terms.resize(cptGetArity(preds.elems[jeu[posPredicate]]));
			    	//initialisation de la table des termes
			    	for(size_t k=0; k<terms.size();k++){
			    		terms[k]=cptGetPossibleTerms(preds.elems[jeu[posPredicate]],k);
			    	}
			    	// choix des termes
			    	std::vector<CPTTerm> argument;
			    	argument.resize(terms.size());
			    	for(size_t k=0;k<argument.size();k++)
			    	{
			    		argument[k]=terms[k].elems[rng.random(terms[k].nb)];
			    	}
			    	
			    	// construction du fluent correspondant 
			    	std::string fluentChar("(");
			    	fluentChar+=cptGetPredicateName(preds.elems[jeu[posPredicate]]);
			    	
			    	for(size_t k=0;k<argument.size();k++)
			    	{
			    		fluentChar+=" ";
			    		fluentChar+=cptGetTermName(argument[k]);
			    		
			    	}
			    	fluentChar+=")";
			    	//cout << fluentChar.c_str();
			    	CPTFluent fluent=cptGetFluentFromName((char *)fluentChar.c_str());
			    	if(fluent!=NULL)
			    	{
			    		
			    		if(station.size()< 100 && station.mutex2(cptGetIndexFromFluent(fluent),station.size())==0
			    		 && station.notExist(cptGetIndexFromFluent(fluent))){
			    		 	station.getGene().push_back(cptGetIndexFromFluent(fluent));
			    		 }
			    		else test=true;
			    		
			    		ho++;
			    	}
			    	else test=true;
				}while(test && !ho==10);
			}
	 	}
			   
			   // fin construction d'une station
			 /*******************************************************
		      *														* 
		      *		 debut de construction du genotype				*
		      * 													*
		      *******************************************************/
		      pos=0;
			   for(size_t i=0;i<tmp.size();i++){
			   		if(i==which){
			   			_genotype.set(pos,station);
			   			pos++;
			   		}
			   		_genotype.set(pos,tmp[i]);
			   		tmp[i].getGene().clear();
			   		pos++;
			   		
			   	}
			   isModified=true;	
			   tmp.clear();
			   station.getGene().clear();
			   
    }

    else {
    	//sauvegarde et initialisation du genotype
    	//cout << "\n *******************which avant**************************** \n " << which;
    	if(which==_genotype.getGenome().size())which=which-1;
    	else which=0;
    	// definition des ensembles mutex, intersection et difference
    	std::vector<int> mutexF;
    	std::vector<int> mutexS;
    	std::vector<int> intersection;
    	std::vector<int> minus;
    	mutexF.resize(_genotype.get(which).size()<global.problemGoal.size()?_genotype.get(which).size():global.problemGoal.size());
		mutexS.resize(_genotype.get(which).size()<global.problemGoal.size()?_genotype.get(which).size():global.problemGoal.size());
		intersection.resize(_genotype.get(which).size()<global.problemGoal.size()?_genotype.get(which).size():global.problemGoal.size());
		minus.resize(global.problemGoal.size());
		//definition et initialisation des variables contenant les tailles reelles des differents ensembles
		int fMutex=0,inter=0,moins=0;
		
    	if(which==0)
    	{
    		
    			//initialisation des differents ensembles
			for(size_t i=0;i<_genotype.get(which).size();i++)
			{	int tester=0;
				for(int j=0;(size_t)j<global.problemGoal.size();j++)
				{
					if(cptMutex(cptGetFluentFromIndex(global.problemInit[j]),cptGetFluentFromIndex(_genotype.get(which).get(i))))
					{
						mutexF[fMutex]=global.problemInit[j];
						mutexS[fMutex]=_genotype.get(which).get(i);
						fMutex++;
						tester=1;
						break;
					}
					else if(global.problemInit[j]==_genotype.get(which).get(i))
					{
						intersection[inter]=_genotype.get(which).get(i);
						inter++;
						tester=1;
						break;
					}
				}
				if(tester==0){
					minus[moins]=_genotype.get(which).get(i);
					moins++;
				}
				
				
			}
			
			
			
    	}
    	else 
    	{
    			//initialisation des differents ensembles
			for(size_t i=0;i<global.problemGoal.size();i++)
			{	int tester=0;
				for(int j=0;(size_t)j<_genotype.get(which).size();j++)
				{
					if(cptMutex(cptGetFluentFromIndex(global.problemGoal[i]),cptGetFluentFromIndex(_genotype.get(which).get(j))))
					{
						mutexF[fMutex]=_genotype.get(which).get(j);
						mutexS[fMutex]=global.problemGoal[i];
						fMutex++;
						tester=1;
						break;
					}
					else if(global.problemGoal[i]==_genotype.get(which).get(j))
					{
						intersection[inter]=_genotype.get(which).get(j);
						inter++;
						tester=1;
						break;
					}
				}
				if(tester==0){
					minus[moins]=global.problemGoal[i];
					moins++;
				}
				
			}
    	}
    	//verification de l'egalite entre les deux stations
			if(fMutex==0 && moins==0) return false;
		//sauvegarde des gares 
    	std::vector<Gene> tmp;
	    tmp.resize(_genotype.getGenome().size());
	    for(size_t i=0;i<tmp.size();i++){
		   		tmp[i].getGene().resize(_genotype.get(i).size());
		   		 for(size_t j=0;j<tmp[i].size();j++){
		   		 	tmp[i].set(j,_genotype.get(i).get(j));
		   		 }
		   		 _genotype.get(i).getGene().clear();
		   	}
	     _genotype.getGenome().clear();
		 _genotype.getGenome().resize(tmp.size()+1);
		 // fin de la reinitialisation et sauvegarde du genotype  
	     /*******************************************************
	      *														* 
	      * debut de construction de la station intermediare	*
	      * 													*
	      *******************************************************/
		   // choix de la taille 
		   for(size_t i=0;i<(size_t)inter;i++){
			   	   		station.getGene().push_back(intersection[i]);
			   }
			   
			 for(size_t i=0;i<(size_t)fMutex;i++)
			   {
			   		if(rng.flip(0.5) && station.mutex2(mutexF[i],station.size())==0
					 && station.notExist(mutexF[i]))station.getGene().push_back(mutexF[i]);
			   		else if(station.mutex2(mutexS[i],station.size())==0
			   		&& station.notExist(mutexS[i])) station.getGene().push_back(mutexS[i]);
			   }  
			     
			 for(size_t i=0;(int)i<moins;i++)
			   {
			   		if(rng.flip(0.5)&& station.mutex2(minus[i],station.size())==0
					 && station.notExist(minus[i]))station.getGene().push_back(minus[i]);
			   		
			   		
			   }    
			
			int ho;
	 	if(rng.flip(global.getpMeanAveragePerMutation()/tmp.size())){
	 		CPTPredicateArray preds = cptGetPredicates();
  			std::vector<unsigned>jeu;
  			jeu.resize(global.getPredicateSize());
  			//initialisation avec la valeur exacte de la table des predicats
  			for(size_t i=0;i<jeu.size();i++) 
  			{	size_t j;
  				for(j=0;j<(unsigned)preds.nb;j++)
  					if(strcmp(cptGetPredicateName(preds.elems[j]),global.getPredicate(i).c_str())==0)break;
  				jeu[i]=j;
  			}
  			unsigned posPredicate, plus=eo::rng.random(global.getDistance());
  			bool test;
  			for(size_t j=0;j<plus;j++)
			{
				ho=0;
				do
				{
					test=false;
					//choix du predicat 
					posPredicate=rng.random(jeu.size());
					//debut tirage aleatoire de termes 
					//	cout << posPredicate << "***************\n";				
					std::vector<CPTTermArray> terms;
			    	terms.resize(cptGetArity(preds.elems[jeu[posPredicate]]));
			    	//initialisation de la table des termes
			    	for(size_t k=0; k<terms.size();k++){
			    		terms[k]=cptGetPossibleTerms(preds.elems[jeu[posPredicate]],k);
			    	}
			    	// choix des termes
			    	std::vector<CPTTerm> argument;
			    	argument.resize(terms.size());
			    	for(size_t k=0;k<argument.size();k++)
			    	{
			    		argument[k]=terms[k].elems[rng.random(terms[k].nb)];
			    	}
			    	
			    	// construction du fluent correspondant 
			    	std::string fluentChar("(");
			    	fluentChar+=cptGetPredicateName(preds.elems[jeu[posPredicate]]);
			    	
			    	for(size_t k=0;k<argument.size();k++)
			    	{
			    		fluentChar+=" ";
			    		fluentChar+=cptGetTermName(argument[k]);
			    		
			    	}
			    	fluentChar+=")";
			    	//cout << fluentChar.c_str();
			    	CPTFluent fluent=cptGetFluentFromName((char *)fluentChar.c_str());
			    	if(fluent!=NULL)
			    	{
			    		
			    		if(station.size()< 100 && station.mutex2(cptGetIndexFromFluent(fluent),station.size())==0
			    		 && station.notExist(cptGetIndexFromFluent(fluent))){
			    		 	station.getGene().push_back(cptGetIndexFromFluent(fluent));
			    		 }
			    		else test=true;
			    		
			    		ho++;
			    	}
			    	else test=true;
				}while(test && !ho==10);
			}
	 	}
		   // fin construction d'une station

		 /*******************************************************
	      *														* 
	      *		 debut de construction du genotype				*
	      * 													*
	      *******************************************************/
	      pos=0;
		   for(size_t i=0;i<tmp.size();i++){
		   		if(i==which){
		   			_genotype.set(pos,station);
		   			pos++;
		   		}
		   		_genotype.set(pos,tmp[i]);
		   		tmp[i].getGene().clear();
		   		pos++;
		   		
		   	}
		   //	tmp.resize(0);
		   isModified=true;	
		   tmp.clear();
			station.getGene().clear();
    }
  
   //  cout << " \n ***************after add mut ****************\n" << _genotype; 
   return  isModified;


}
};
#endif /*AJOUTDESTATIONSANSINIT_H_*/
